#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : runpar.py
# Contents: simple parallelization script for a single machine
# Author  : Christian Borgelt
# History : 2009.08.22 file created
#           2009.08.24 adapted to subprocess module
#           2009.09.01 adapted to argument ranges starting at 1
#           2011.08.18 changed to process status polling
#-----------------------------------------------------------------------
from sys        import argv, exit
from time       import sleep
from subprocess import Popen, PIPE, check_output

if len(argv) <= 1:              # print a usage message if no arguments
    print 'usage:  ' +argv[0] +' prog min max cpus wait'
    print 'prog   program or script to run'
    print 'min    minimum argument (integer number, default: 1)'
    print 'max    maximum argument (integer number, default: from prog)'
    print 'cpus   number of CPUs to use            (default: 4)'
    print 'wait   time for process status polling  (default: 0.2s)'
    exit()                      # terminate the program after printing

prog = argv[1]                  # get the program arguments
beg  = int(argv[2]) if len(argv) > 2 else  1
end  = int(argv[3]) if len(argv) > 3 else -1
cpus = int(argv[4]) if len(argv) > 4 else  4
wait = int(argv[5]) if len(argv) > 5 else  0.2

if end  <  0: end  = int(check_output([prog]))
if cpus <= 0: cpus = 1          # get the end of the argument range

print 'running', prog, 'for range [' +str(beg) +'..' +str(end) +']'
procs = []; arg = beg           # while there are arguments left
while arg <= end or len(procs) > 0: # or processes still running
    while arg <= end and len(procs) < cpus:
        procs.append(Popen([prog, str(arg)]))
        arg += 1                # start a new process if slot is free
    sleep(wait)                 # wait for processes to do some work
    for p in procs: p.poll()    # collect unterminated processes
    procs = [p for p in procs if p.returncode == None]
print 'finished', prog, 'for range [' +str(beg) +'..' +str(end) +']'
